package com.mortals.xhx.module.certificate.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.Context;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.annotation.DataPermission;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.module.certificate.model.CertificateIndustryEntity;
import com.mortals.xhx.module.certificate.model.CertificateIndustryQuery;
import com.mortals.xhx.module.certificate.model.CertificateIndustryTreeSelect;
import com.mortals.xhx.module.certificate.service.CertificateIndustryService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

import static com.mortals.framework.ap.SysConstains.RESULT_KEY;

/**
 * 行业目录
 *
 * @author zxfei
 * @date 2022-10-14
 */
@RestController
@RequestMapping("certificate/industry")
public class CertificateIndustryController extends BaseCRUDJsonBodyMappingController<CertificateIndustryService, CertificateIndustryEntity, Long> {

    @Autowired
    private ParamService paramService;

    public CertificateIndustryController() {
        super.setModuleDesc("行业目录");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        super.init(model, context);
    }

    @Override
    @DataPermission
    public Rest<Object> list(CertificateIndustryEntity query) {
        return super.list(query);
    }

    @PostMapping({"treeList"})
    @UnAuth
    public Rest<Object> treeList(@RequestBody CertificateIndustryEntity query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();
        int code;
        try {
            List<CertificateIndustryEntity> result = this.getService().find(query, context);
            if (CollectionUtils.isNotEmpty(result)) {
                List<CertificateIndustryEntity> collect = result.stream().filter(t -> t.getParentId() == -1).map(
                        m -> {
                            m.setChildren(getChildren(m, result));
                            return m;
                        }
                ).collect(Collectors.toList());
                model.put("data", collect);
            } else {
                model.put("data", result);
            }
            code = this.doListAfter(query, (Map) model, context);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    private List<CertificateIndustryEntity> getChildren(CertificateIndustryEntity root, List<CertificateIndustryEntity> all) {
        List<CertificateIndustryEntity> children = all.stream().filter(t -> {
            return Objects.equals(t.getParentId(), root.getId());
        }).map(
                m -> {
                    m.setChildren(getChildren(m, all));
                    return m;
                }
        ).collect(Collectors.toList());
        return children;
    }



    /**
     * 获取站点下拉树列表
     */
    @GetMapping("treeselect")
    @UnAuth
    public String treeselect() {
        Map<String, Object> model = new HashMap<>();
        JSONObject ret = new JSONObject();
        String busiDesc = "查询" + this.getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            List<CertificateIndustryEntity> list = this.service.find(new CertificateIndustryQuery());
            List<CertificateIndustryTreeSelect> treeSelects = this.service.buildCertificateIndustryTreeSelect(list);
            model.put("data", treeSelects);
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        ret.put(KEY_RESULT_DATA, model);
        ret.put(KEY_RESULT_CODE, code);
        return ret.toJSONString();
    }


    /**
     * 根据parentId查询子信息
     */
    @GetMapping(value = "getListByParentId")
    public String getListByParentId(Long parentId) {
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = "查询" + this.getModuleDesc() + "子节点";
        try {
            List<CertificateIndustryTreeSelect> treeList = this.service.getListByParentId(parentId, getContext());
            model.put(RESULT_KEY, treeList);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            ret.put(KEY_RESULT_DATA, model);
            recordSysLog(request, busiDesc + "【成功】");
        } catch (Exception e) {
            log.error("根据parentId查询子信息错误", e);
            this.doException(request, busiDesc, model, e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        return ret.toJSONString();
    }


}